import { getInfo, login, logout } from '@/api/login'
import { getToken, removeToken, setToken } from '@/utils/auth'
import {
  getApplicationId,
  removeApplicationId,
  removeApplicationName,
  setApplicationId,
  setApplicationName
} from '@/utils/application'
import Cookies from 'js-cookie'

const user = {
  state: {
    token: getToken(),
    name: '',
    avatar: '',
    roles: [],
    permissions: [],
    applicationId: getApplicationId(),
    applicationInfo: {} // 用户选中的项目/应用
  },

  mutations: {
    SET_TOKEN: (state, token) => {
      state.token = token
    },
    SET_NAME: (state, name) => {
      state.name = name
    },
    SET_AVATAR: (state, avatar) => {
      state.avatar = avatar
    },
    SET_ROLES: (state, roles) => {
      state.roles = roles
    },
    SET_PERMISSIONS: (state, permissions) => {
      state.permissions = permissions
    },
    SET_APPLICATION: (state, applicationInfo) => {
      const { appCode } = applicationInfo
      state.applicationId = appCode || ''
      state.applicationInfo = applicationInfo
    }
  },

  actions: {
    // 登录
    Login ({ commit }, userInfo) {
      const username = userInfo.username.trim()
      const password = userInfo.password
      const code = userInfo.code
      const uuid = userInfo.uuid
      return new Promise((resolve, reject) => {
        login(username, password, code, uuid).then(res => {
          setToken(res.token)
          commit('SET_TOKEN', res.token)
          resolve()
        }).catch(error => {
          reject(error)
        })
      })
    },

    // 获取用户信息
    GetInfo ({ commit, state }) {
      return new Promise((resolve, reject) => {
        getInfo().then(res => {
          const user = res.user
          const avatar = (user.avatar == '' || user.avatar == null) ? require('@/assets/images/profile.png') : process.env.VUE_APP_BASE_API + user.avatar
          if (res.roles && res.roles.length > 0) { // 验证返回的roles是否是一个非空数组
            commit('SET_ROLES', res.roles)
            commit('SET_PERMISSIONS', res.permissions)
          } else {
            commit('SET_ROLES', ['ROLE_DEFAULT'])
          }
          commit('SET_NAME', user.userName)
          commit('SET_AVATAR', avatar)
          resolve(res)
        }).catch(error => {
          reject(error)
        })
      })
    },

    // 退出系统
    LogOut ({ commit, state }) {
      return new Promise((resolve, reject) => {
        logout(state.token).then(() => {
          commit('SET_TOKEN', '')
          commit('SET_ROLES', [])
          commit('SET_PERMISSIONS', [])
          commit('SET_APPLICATION', {})
          removeApplicationId()
          removeToken()
          resolve()
        }).catch(error => {
          reject(error)
        })
      })
    },

    // 前端 登出
    FedLogOut ({ commit }) {
      return new Promise(resolve => {
        commit('SET_TOKEN', '')
        commit('SET_APPLICATION', {})
        removeApplicationId()
        removeToken()
        resolve()
      })
    },

    // 设置选中的项目应用信息
    SetApplication ({ commit }, applicationInfo) {
      commit('SET_APPLICATION', applicationInfo)
      const { appCode, appName, isAdmin } = applicationInfo || {}
      if (appCode) {
        setApplicationId(appCode)
        Cookies.set('Admin-Application-appCode', applicationInfo.appCode)
      } else {
        removeApplicationId()
        Cookies.remove('Admin-Application-appCode')
      }
      if (appName) {
        setApplicationName(applicationInfo.appName)
      } else {
        removeApplicationName()
      }


      if (isAdmin) {
        Cookies.set('Admin-isAdmin', isAdmin)
      } else {
        Cookies.set('Admin-isAdmin', 0)
      }

    }
  }
}

export default user
